//-----------------------------------------------------------------------------
// F3xx_USB0_ReportHandler.c
//-----------------------------------------------------------------------------
// Copyright 2005 Silicon Laboratories, Inc.
// http://www.silabs.com
//
// Program Description:
//
// Contains functions and variables dealing with Input and Output
// HID reports.
// How To Test:    See Readme.txt
//
//
// FID:
// Target:         C8051F3xx
// Tool chain:     Keil C51 7.50 / Keil EVAL C51
//                 Silicon Laboratories IDE version 2.6
// Command Line:   See Readme.txt
// Project Name:   F3xx_FirmwareTemplate
//
// Release 1.1
//    - Minor changes to F3xx_USB0_Descriptor.c
//    - 16 NOV 2006
// Release 1.0
//    -Initial Revision (PD)
//    -07 DEC 2005
//


// ----------------------------------------------------------------------------
// Header files
// ----------------------------------------------------------------------------

#include "precomp.h"

// ----------------------------------------------------------------------------
// Local Function Prototypes
// ----------------------------------------------------------------------------

// ****************************************************************************
// Add custom Report Handler Prototypes Here
// ****************************************************************************
static void Report1Handler(void);
static void Report2Handler(void);
static void Report3Handler(void);
static void Report4Handler_IN(void);
static void Report4Handler_OUT(void);
static void Report5Handler(void);
static void Report6Handler(void);
static void Report7Handler_IN(void);
static void Report7Handler_OUT(void);
static void Report8Handler(void);

// ----------------------------------------------------------------------------
// Local Definitions
// ----------------------------------------------------------------------------

static xdata IO_REPORT input_report;
static xdata IO_REPORT output_report;
static BOOL gs_blMemoryPreserved;

// ----------------------------------------------------------------------------
// Global Constant Declaration
// ----------------------------------------------------------------------------


// ****************************************************************************
// Link all Report Handler functions to corresponding Report IDs
// ****************************************************************************

idata const VectorTableEntry IN_VECTORTABLE[] =
{
   // FORMAT: Report ID, Report Handler
   {    1, Report1Handler       },
   {    4, Report4Handler_IN    },
   {    5, Report5Handler       },
   {    6, Report6Handler       },
   {    7, Report7Handler_IN    },
   {    8, Report8Handler       }
};

// ****************************************************************************
// Link all Report Handler functions to corresponding Report IDs
// ****************************************************************************
const VectorTableEntry OUT_VECTORTABLE[] =
{
   // FORMAT: Report ID, Report Handler
    {   2, Report2Handler       },
    {   3, Report3Handler       },
    {   4, Report4Handler_OUT   },
    {   7, Report7Handler_OUT   }
};


// ----------------------------------------------------------------------------
// Global Variable Declaration
// ----------------------------------------------------------------------------

BufferStructure IN_BUFFER, OUT_BUFFER;
static BYTE gs_bLastRecentResult;

// ----------------------------------------------------------------------------
// Local Variable Declaration
// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// Local Functions
// ----------------------------------------------------------------------------

// ****************************************************************************
// Add all Report Handler routines here
// ****************************************************************************


// ****************************************************************************
// For Input Reports:
// Point IN_BUFFER.Ptr to the buffer containing the report
// Set IN_BUFFER.Length to the number of bytes that will be
// transmitted.
//
// REMINDER:
// Systems using more than one report must define Report IDs
// for each report.  These Report IDs must be included as the first
// byte of an IN report.
// Systems with Report IDs should set the FIRST byte of their buffer
// to the value for the Report ID
// AND
// must transmit Report Size + 1 to include the full report PLUS
// the Report ID.
//
// ****************************************************************************



// ****************************************************************************
// For Output Reports:
// Data contained in the buffer OUT_BUFFER.Ptr will not be
// preserved after the Report Handler exits.
// Any data that needs to be preserved should be copyed from
// the OUT_BUFFER.Ptr and into other user-defined memory.
//
// ****************************************************************************


// ----------------------------------------------------------------------------
// Global Functions
// ----------------------------------------------------------------------------


// ****************************************************************************
// Configure Setup_OUT_BUFFER
//
// Reminder:
// This function should set OUT_BUFFER.Ptr so that it
// points to an array in data space big enough to store
// any output report.
// It should also set OUT_BUFFER.Length to the size of
// this buffer.
//
// ****************************************************************************

void Setup_IN_BUFFER(void)
{
   IN_BUFFER.Ptr = (unsigned char*)&input_report;
}

void Setup_OUT_BUFFER(void)
{
    OUT_BUFFER.Ptr = (unsigned char*)&output_report;
    OUT_BUFFER.Length = sizeof(output_report);
}

// ----------------------------------------------------------------------------
// Vector Routines
// ----------------------------------------------------------------------------


// ----------------------------------------------------------------------------
// ReportHandler_IN...
// ----------------------------------------------------------------------------
//
// Return Value - None
// Parameters - Report ID
//
// These functions match the Report ID passed as a parameter
// to an Input Report Handler.
// the ...FG function is called in the SendPacket foreground routine,
// while the ...ISR function is called inside the USB ISR.  A lock
// is set whenever one function is called to prevent a call from the
// other from disrupting the routine.
// However, this should never occur, as interrupts are disabled by SendPacket
// before USB operation begins.
// ----------------------------------------------------------------------------
void ReportHandler_IN_ISR(unsigned char R_ID)
{
   unsigned char index;

   index = 0;

   while(index < _countof(IN_VECTORTABLE))
   {
      // Check to see if Report ID passed into function
  	  // matches the Report ID for this entry in the Vector Table
      if(IN_VECTORTABLE[index].ReportID == R_ID)
      {
         IN_VECTORTABLE[index].hdlr();
         break;
      }

      // If Report IDs didn't match, increment the index pointer
      index++;
   }

}
void ReportHandler_IN_Foreground(unsigned char R_ID)
{
   unsigned char index;

   index = 0;

   while(index < _countof(IN_VECTORTABLE))
   {
      // Check to see if Report ID passed into function
      // matches the Report ID for this entry in the Vector Table
      if(IN_VECTORTABLE[index].ReportID == R_ID)
      {
         IN_VECTORTABLE[index].hdlr();
         break;
      }

      // If Report IDs didn't match, increment the index pointer
      index++;
   }

}

// ----------------------------------------------------------------------------
// ReportHandler_OUT
// ----------------------------------------------------------------------------
//
// Return Value - None
// Parameters - None
//
// This function matches the Report ID passed as a parameter
// to an Output Report Handler.
//
// ----------------------------------------------------------------------------
void ReportHandler_OUT(unsigned char R_ID){

   unsigned char index;

   index = 0;

   while(index < _countof(OUT_VECTORTABLE))
   {
      // Check to see if Report ID passed into function
      // matches the Report ID for this entry in the Vector Table
      if(OUT_VECTORTABLE[index].ReportID == R_ID)
      {
         OUT_VECTORTABLE[index].hdlr();
         break;
      }

      // If Report IDs didn't match, increment the index pointer
      index++;
   }
}

static void Report1Handler(void)
{
    WORD w;
    w.i = DbgItf_GetChipId();

    input_report.bReportID = 1;
    input_report.dat.report1.bChipId = w.c[MSB];
    input_report.dat.report1.bVersion = w.c[LSB];
    input_report.dat.report1.bStatus = DbgItf_GetStatus();
    EA = 0;
    input_report.dat.report1.bTargetVoltageL = g_wTargetVoltage.c[LSB];
    input_report.dat.report1.bTargetVoltageH = g_wTargetVoltage.c[MSB];
    EA = 1;
    IN_BUFFER.Length = sizeof(REPORT1) + 1;
}

static void Report2Handler(void)
{
    switch(output_report.dat.report2.bCmd)
    {
        case CMDID_DEBUGMODE:
            if(output_report.dat.report2.param1.i)
            {
                DbgItf_EnterDebugMode();
            }
            else
            {
                if(gs_blMemoryPreserved)
                {
                    Memory_Restore();
                    gs_blMemoryPreserved = FALSE;
                }
                DbgItf_LeaveDebugMode();
            }

            break;
        case CMDID_DEBUGCONFIG:
            DbgItf_WriteDebugConfig(output_report.dat.report2.param1.c[0]);
            break;
        case CMDID_RESUME:
            if(gs_blMemoryPreserved)
            {
                Memory_Restore();
                gs_blMemoryPreserved = FALSE;
            }
            DbgItf_Resume();
            break;
        case CMDID_HALT:
            DbgItf_Halt();
            break;
        case CMDID_CHIP_ERASE:
            DbgItf_ChipErase();
            break;
        case CMDID_DEBUG_INSTR:
            DbgItf_DebugInstr(output_report.dat.report2.param2.c[1], output_report.dat.report2.param1.c[0], output_report.dat.report2.param1.c[1], output_report.dat.report2.param2.c[0], &gs_bLastRecentResult);
            break;
        case CMDID_STEP_REPLACE:
            DbgItf_StepReplace(output_report.dat.report2.param2.c[1], output_report.dat.report2.param1.c[0], output_report.dat.report2.param1.c[1], output_report.dat.report2.param2.c[0], &gs_bLastRecentResult);
            break;
        case CMDID_STEP_INSTR:
            DbgItf_StepInstr(&gs_bLastRecentResult);
            break;
        case CMDID_SET_HW_BRKPNT:
            DbgItf_SetHwBreakpoint((output_report.dat.report2.param2.c[1] & 0x7F), ((output_report.dat.report2.param2.c[1] & 0x80) != 0), MAKEDWORD(LENDIAN_TO_IMPL(output_report.dat.report2.param1.i), output_report.dat.report2.param2.c[0]));
            break;
        default:
            break;
    }
}

static void Report3Handler(void)
{
    g_memInfo.bSegment  = output_report.dat.report3.bSegment;
    g_memInfo.uAddress  = ntohl(output_report.dat.report3.uAddress);
    g_memInfo.uLen      = ntohl(output_report.dat.report3.uLen);
    g_memInfo.blRead    = output_report.dat.report3.blRead;

    if((gs_blMemoryPreserved) && (g_memInfo.bSegment != segCode))
    {
        Memory_Restore();
        gs_blMemoryPreserved = FALSE;
    }
}

static void Report4Handler_IN(void)
{
    BOOL blEA = EA;
    input_report.bReportID = 4;
    IN_BUFFER.Length = sizeof(REPORT4) + 1;

    if(!DbgItf_IsInDebugMode())
    {
        input_report.dat.report4.bLength = 0xFF;
        EA = 0;
        g_memInfo.uLen = 0;
        EA = blEA;
    }
    else
    {
        EA = 0;

        if(g_memInfo.uLen >= sizeof(input_report.dat.report4.bData))
        {
            g_memInfo.uLen -= sizeof(input_report.dat.report4.bData);
            input_report.dat.report4.bLength = sizeof(input_report.dat.report4.bData);
        }
        else
        {
            input_report.dat.report4.bLength = (BYTE)g_memInfo.uLen;
            g_memInfo.uLen = 0;
        }

        EA = blEA;

        if(input_report.dat.report4.bLength)
        {
            switch(g_memInfo.bSegment)
            {
                case segCode:
                    Memory_ReadCode(g_memInfo.uAddress, input_report.dat.report4.bLength, input_report.dat.report4.bData);
                    break;
                case segData:
                    Memory_ReadData(LOBYTE(g_memInfo.uAddress), input_report.dat.report4.bLength, input_report.dat.report4.bData);
                    break;
                case segXData:
                    Memory_ReadXData(LOWORD(g_memInfo.uAddress), input_report.dat.report4.bLength, input_report.dat.report4.bData);
                    break;
                case segIData:
                    Memory_ReadIData(LOBYTE(g_memInfo.uAddress), input_report.dat.report4.bLength, input_report.dat.report4.bData);
                    break;
                default:
                    return;
            }
        }

        EA = 0;
        g_memInfo.uAddress += input_report.dat.report4.bLength;
        EA = blEA;
    }
}

static void Report4Handler_OUT(void)
{
    BOOL blEA = EA;

    if(output_report.dat.report4.bLength)
    {
        switch(g_memInfo.bSegment)
        {
            case segCode:
                if(!gs_blMemoryPreserved)
                    gs_blMemoryPreserved = Memory_Preserve();

                if(gs_blMemoryPreserved)
                {
                    Memory_WriteCode(g_memInfo.uAddress, output_report.dat.report4.bLength, output_report.dat.report4.bData);
                    g_memInfo.uLen -= output_report.dat.report4.bLength;

                    if(!g_memInfo.uLen)
                    {
                        Memory_Restore();
                        gs_blMemoryPreserved = FALSE;
                    }
                }
                break;
            case segData:
                Memory_WriteData(LOBYTE(g_memInfo.uAddress), output_report.dat.report4.bLength, output_report.dat.report4.bData);
                break;
            case segXData:
                Memory_WriteXData(LOWORD(g_memInfo.uAddress), output_report.dat.report4.bLength, output_report.dat.report4.bData);
                break;
            case segIData:
                Memory_WriteIData(LOBYTE(g_memInfo.uAddress), output_report.dat.report4.bLength, output_report.dat.report4.bData);
                break;
            default:
                return;
        }

        EA = 0;
        g_memInfo.uAddress += output_report.dat.report4.bLength;
        EA = blEA;
    }
}

static void Report5Handler(void)
{
    input_report.bReportID = 5;
    input_report.dat.report5.bInDbgMode = (DbgItf_ReadDebugConfig(&input_report.dat.report5.bConfig) && DbgItf_ReadDebugStatus(&input_report.dat.report5.bStatus)) ? 0xFF : 0x00;
    input_report.dat.report5.bResult = gs_bLastRecentResult;
    IN_BUFFER.Length = sizeof(REPORT5) + 1;
}

static void Report6Handler(void)
{
    if(!DbgItf_GetPc(&input_report.dat.report6.pc))
    {
        input_report.dat.report6.pc = IMPL_TO_LENDIAN(0);
    }
    else
    {
        input_report.dat.report6.pc = IMPL_TO_LENDIAN(input_report.dat.report6.pc);
    }

    input_report.bReportID = 6;
    IN_BUFFER.Length = sizeof(REPORT6) + 1;
}

static void Report7Handler_IN(void)
{
    memset(&input_report.dat.report7, 0xFF, sizeof(input_report.dat.report7));
    Memory_ReadLockBits(input_report.dat.report7.bLockBits);
    input_report.bReportID = 7;
    IN_BUFFER.Length = sizeof(REPORT7) + 1;
}

static void Report7Handler_OUT(void)
{
    Memory_WriteLockBits(output_report.dat.report7.bLockBits);
}

static void Report8Handler(void)
{
    input_report.bReportID = 8;

    if(!DbgItf_GetBm(&input_report.dat.report8.bBank))
        input_report.dat.report8.bBank = 0;

    IN_BUFFER.Length = sizeof(REPORT8) + 1;
}
