#include "precomp.h"

CEnumConnectionPoints::CEnumConnectionPoints() throw()
    : m_lRefCount(1), m_lCursor(0)
{
}

CEnumConnectionPoints::~CEnumConnectionPoints() throw()
{
    IConnectionPoint* pCP;

    while(m_list.Remove(0, &pCP))
        pCP->Release();
}

STDMETHODIMP
CEnumConnectionPoints::QueryInterface(THIS_ REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = static_cast<IUnknown*>(this);
        else if(IsEqualIID(riid, IID_IEnumConnectionPoints))
            *ppvObject = static_cast<IEnumConnectionPoints*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CEnumConnectionPoints::AddRef(THIS)
{
    return(InterlockedIncrement(&m_lRefCount));
}

STDMETHODIMP_(ULONG)
CEnumConnectionPoints::Release(THIS)
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

STDMETHODIMP
CEnumConnectionPoints::Next(THIS_ ULONG cConnections, IConnectionPoint** ppCP, ULONG* pcFetched)
{
    ULONG_PTR   lCursor;
    ULONG       ulCnt;
    BOOL        bl;

    if(FALSE == (bl = IsBadWritePtrAuto(pcFetched)))
        *pcFetched = 0;

#ifdef _DEBUG
    if(IsBadWritePtr(ppCP, (cConnections * sizeof(*ppCP))))
#else
    if(!ppCP)
#endif
        return(E_POINTER);

    ZeroMemory(ppCP, (cConnections * sizeof(*ppCP)));

    if(bl)
    {
        if((cConnections > 1) || (pcFetched))
            return(E_INVALIDARG);

        ulCnt = 0;
        pcFetched = &ulCnt;
        bl = FALSE;
    }

    for(; ((*pcFetched < cConnections) && (!bl)); ++(*pcFetched), ++ppCP)
    {
        do
        {
            if(*ppCP)
            {
                (*ppCP)->Release();
                *ppCP = NULL;
            }

#ifndef _MSC_VER
            lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
            // Microsoft uses Acquire semantics on volatile variables.
            lCursor = m_lCursor;
#endif

            if(!m_list.Get(lCursor, ppCP))
            {
                --(*pcFetched);
                bl = TRUE;
                break;
            }

        } while(InterlockedCompareExchangeRelease(&m_lCursor, lCursor + 1, lCursor) != lCursor);
    }

    return((*pcFetched == cConnections) ? S_OK : S_FALSE);
}

STDMETHODIMP
CEnumConnectionPoints::Skip(THIS_ ULONG cConnections)
{
    ULONG_PTR   lCurrentValue, lNewValue;
    HRESULT     hr;

    do
    {
#ifndef _MSC_VER
        lCurrentValue = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
        // Microsoft uses Acquire semantics on volatile variables.
        lCurrentValue = m_lCursor;
#endif

        lNewValue = lCurrentValue + cConnections;

        if(lNewValue > m_list.Size())
        {
            lNewValue = m_list.Size();
            hr = S_FALSE;
        }
        else
        {
            hr = S_OK;
        }

    } while(InterlockedCompareExchangeRelease(&m_lCursor, lNewValue, lCurrentValue) != lCurrentValue);

    return(hr);
}

STDMETHODIMP
CEnumConnectionPoints::Reset(THIS)
{
    InterlockedExchange(&m_lCursor, 0);
    return(S_OK);
}

STDMETHODIMP
CEnumConnectionPoints::Clone(THIS_ IEnumConnectionPoints** ppEnum)
{
    CEnumConnectionPoints*  pEnum = NULL;
    IConnectionPoint*       pCP;
    UINT_PTR                x;
    HRESULT                 hr = S_OK;

    if(IsBadWritePtrAuto(ppEnum))
        return(E_POINTER);

    *ppEnum = NULL;

    if(NULL == (pEnum = new (std::nothrow) CEnumConnectionPoints))
        return(E_OUTOFMEMORY);

    for(x = 0; m_list.Get(x, &pCP); ++x)
    {
        if(FAILED((hr = pEnum->Add(pCP))))
        {
            if(hr != E_OUTOFMEMORY)
                hr = E_UNEXPECTED;

            break;
        }
    }

    if(SUCCEEDED(hr))
    {
#ifndef _MSC_VER
        pEnum->m_lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
        // Microsoft uses Acquire semantics on volatile variables.
        pEnum->m_lCursor = m_lCursor;
#endif

        hr = pEnum->QueryInterface(IID_IEnumConnectionPoints, (void**)ppEnum);
    }

    pEnum->Release();
    return(hr);
}

HRESULT
CEnumConnectionPoints::Add(IUnknown* pCPt) throw()
{
    IConnectionPoint*   pCP = NULL;
    HRESULT             hr;

    if(FAILED((hr = pCPt->QueryInterface(IID_IConnectionPoint, (void**)&pCP))))
        return(hr);

    if(!m_list.Add(pCP))
    {
        pCP->Release();
        return(E_OUTOFMEMORY);
    }

    return(S_OK);
}
